/*
 * Log Levels: 
    debug   DEBUG: Info useful to developers for debugging the application, not useful during operations
    info    INFORMATIONAL: Normal operational messages - may be harvested for reporting, measuring throughput, etc - no action required
    notice  NOTICE: Events that are unusual but not error conditions - might be summarized in an email to developers or admins to spot potential problems - no immediate action required
    warning WARNING: Warning messages - not an error, but indication that an error will occur if action is not taken, e.g. file system 85% full - each item must be resolved within a given time
    error   ERROR: Non-urgent failures - these should be relayed to developers or admins; each item must be resolved within a given time
    alert   ALERT: Should be corrected immediately - notify staff who can fix the problem - example is loss of backup ISP connection
    crit    CRITICAL: Should be corrected immediately, but indicates failure in a primary system - fix CRITICAL problems before ALERT - example is loss of primary ISP connection
    emerg   EMERGENCY: A "panic" condition - notify all tech staff on call? (earthquake? tornado?) - affects multiple apps/servers/sites...
 
  Log format:
    '%(levelname)s %(appname)s %(instance)s [%(filename)s:%(lineno)d] %(message)s'
 */ 

/*
 * UWAGA!!!! - robienie console.log w tym pliku może grozic rekurencyjnym wywolaniem - jeśli LogConsole zostało użyte do przeciązenia console
 */

var Dgram = require("dgram");
var Glossy = require("glossy");
var Os = require("os");
 
var socket = Dgram.createSocket("unix_dgram"); 
 
var LogConsole = function(appName, instanceId, version){
    var appName = appName;
    var instanceId = instanceId;
    var appVersion = version;
    var appId = appName + "@" + appVersion + ":" + instanceId;
    var infoTypes = ['info', 'notice'];
    var errorTypes = ['error', 'alert', 'crit', 'emerg'];
    
    var logger = new Glossy.Produce({
        type: 'RFC5424',
        facility: 'local0'//, //domyslne
        //app_id: instanceId,
        //pid: process.pid
    });
        
    //debug
    this.log = function(){
        this._log('debug', arguments);
    };
    //info
    //notice
    this.info = function(){
        this._logTypes(arguments, infoTypes);
    };
    //warning
    this.warn = function(){
        this._log('warning', arguments);
    };
    //error
    //alert
    //crit
    //emerg
    this.error = function(){
        this._logTypes(arguments, errorTypes);
    };
    
    //not implemented
    this.dir = function(){};
    this.time = function(){};
    this.timeEnd = function(){};
    this.trace = function(){};
    this.assert = function(){};
    
    this._formatArguments = function(args){
        var result = "";
        for(var arg in args){
            if(args.hasOwnProperty(arg)){
                result += args[arg] + " ";    
            }   
        }
        return result
    };
    this._log = function(level, message){
        message = this._formatArguments(message);
        message = logger.produce({
            facility: 'local0', //domyslne
            severity: level,
            host: Os.hostname(),
            app_id: appId,
            pid: process.pid,
            msg_id: "",
            message: message
        });
        var data = new Buffer(message);
        socket.send(data, 0, data.length, "/dev/log");
    };
    this._logTypes = function(args, types){
        args = Array.prototype.slice.call(args)
        if(args.length > 1 && typeof(args[0]) == 'string'){
            if(types.indexOf(args[0]) >= 0){
                var newArgs = args.slice(1, args.length); 
                this._log(args[0], newArgs);
                return;
            }
        }
        this._log(types[0], args);        
    };
};
exports.LogConsole = LogConsole;